%module dnet

%{
#include "dnet.h"
%}

#define uint16_t unsigned short
#define uint_8_t unsigned char     
#define uint_32_t unsigned int
#define ip_addr_t unsigned int
#define size_t unsigned int

%inline %{
  struct foo {
      uint16_t	one;
      uint16_t	two;
  };
%}


typedef struct eth_addr {
        unsigned char         data[ETH_ADDR_LEN];
} eth_addr_t;

struct addr {
             uint16_t                addr_type;
             uint16_t                addr_bits;
             union {
                     eth_addr_t      __eth;
                     ip_addr_t       __ip;

                     uint8_t         __data8[20];
                     uint16_t        __data16[10];
                     uint32_t        __data32[5];
             } __addr_u;
     };



     struct arp_entry {
             struct addr     arp_pa;         /* protocol address */
             struct addr     arp_ha;         /* hardware address */
     };



     typedef struct blob {
             u_char          *base;          /* start of data */
             int              off;           /* offset into data */
             int              end;           /* end of data */
             int              size;          /* size of allocation */
     } blob_t;


     struct intf_entry {
             u_int           intf_len;               /* length of entry */
             char            intf_name[60];          /* interface name */
             u_short         intf_type;              /* interface type (r/o) */
             u_short         intf_flags;             /* interface flags */
             u_int           intf_mtu;               /* interface MTU */
             struct addr     intf_addr;              /* interface address */
             struct addr     intf_dst_addr;          /* point-to-point dst */
             struct addr     intf_link_addr;         /* link-layer address */
             u_int           intf_alias_num;         /* number of aliases */
             struct addr     intf_alias_addrs[1] ;    /* array of aliases */
     };


     struct route_entry {
             struct addr     route_dst;      /* destination address */
             struct addr     route_gw;       /* gateway address */
     };

     int     addr_cmp(const struct addr *a, const struct addr *b);

     int     addr_bcast(const struct addr *a, struct addr *b);

     int     addr_ntop(const struct addr *src, char *dst, size_t size);

     int     addr_pton(const char *src, struct addr *dst);

     char *  addr_ntoa(const struct addr *a);

     int     addr_aton(const char *src, struct addr *dst);

     int     addr_ntos(const struct addr *a, struct sockaddr *sa);

     int     addr_ston(const struct sockaddr *sa, struct addr *a);

     int     addr_btos(uint16_t bits, struct sockaddr *sa);

     int     addr_stob(const struct sockaddr *sa, uint16_t *bits);

     int     addr_btom(uint16_t bits, void *mask, size_t size);

     int     addr_mtob(const void *mask, size_t size, uint16_t *bits);

     typedef int (*arp_handler)(const struct arp_entry *entry, void *arg);

     arp_t * arp_open(void);

     int     arp_add(arp_t *a, const struct arp_entry *entry);

     int     arp_delete(arp_t *a, const struct arp_entry *entry);

     int     arp_get(arp_t *a, struct arp_entry *entry);

     int     arp_loop(arp_t *a, arp_handler callback, void *arg);

     arp_t * arp_close(arp_t *a);

  blob_t *   blob_new(void);

     int     blob_read(blob_t *b, void *buf, int len);

     int     blob_write(blob_t *b, const void *buf, int len);

     int     blob_seek(blob_t *b, int off, int whence);

     int     blob_index(blob_t *b, const void *buf, int len);

     int     blob_rindex(blob_t *b, const void *buf, int len);

     int     blob_print(blob_t *b, char *style, int len);

     blob_t *blob_free(blob_t *b);

    eth_t *  eth_open(const char *device);

     int     eth_get(eth_t *e, eth_addr_t *ea);

     int     eth_set(eth_t *e, const eth_addr_t *ea);

     ssize_t  eth_send(eth_t *e, const void *buf, size_t len);

     eth_t *  eth_close(eth_t *e);

     typedef int (*fw_handler)(const struct fw_rule *rule, void *arg);

     fw_t *  fw_open(void);

     int     fw_add(fw_t *f, const struct fw_rule *rule);

     int     fw_delete(fw_t *f, const struct fw_rule *rule);

     int     fw_loop(fw_t *f, fw_handler callback, void *arg);

     fw_t *  fw_close(fw_t *f);

     typedef int (*intf_handler)(const struct intf_entry *entry, void *arg);

     intf_t * intf_open(void);

     int     intf_get(intf_t *i, struct intf_entry *entry);

     int     intf_get_src(intf_t *i, struct intf_entry *entry, struct addr *src);

     int     intf_get_dst(intf_t *i, struct intf_entry *entry, struct addr *dst);

     int     intf_set(intf_t *i, const struct intf_entry *entry);

     int     intf_loop(intf_t *i, intf_handler callback, void *arg);

     intf_t * intf_close(intf_t *i);


     ip_t * ip_open(void);

     size_t ip_add_option(void *buf, size_t len, int proto, const void *optbuf,
             size_t optlen);

     void ip_checksum(void *buf, size_t len);

     ssize_t ip_send(ip_t *i, const void *buf, size_t len);

     ip_t *  ip_close(ip_t *i);

     rand_t * rand_open(void);

     int    rand_get(rand_t *r, void *buf, size_t len);

     int     rand_set(rand_t *r, const void *seed, size_t len);

     int     rand_add(rand_t *r, const void *buf, size_t len);

     uint8_t rand_uint8(rand_t *r);

     uint16_t rand_uint16(rand_t *r);

     uint32_t rand_uint32(rand_t *r);

     int     rand_shuffle(rand_t *r, void *base, size_t nmemb, size_t size);

     rand_t * rand_close(rand_t *r);

     typedef int (*route_handler)(const struct route_entry *entry, void *arg);

     route_t * route_open(void);

     int     route_add(route_t *r, const struct route_entry *entry);

     int     route_delete(route_t *r, const struct route_entry *entry);

     int     route_get(route_t *r, struct route_entry *entry);

     int     route_loop(route_t *r, route_handler callback, void *arg);

     route_t * route_close(route_t *r);

%inline %{

unsigned short *new_uint_16Array(int size) {
return (unsigned short *) malloc(size*sizeof(unsigned short));
}
void delete_uint_16Array(unsigned short *a) {
free (a);
}
unsigned short get_uint_16Array(unsigned short *a, int index) {
return a[index];
}
void set_uint_16Array(unsigned short *a, int index, unsigned short val) {
a[index] = val;
}

unsigned char *new_uint_8Array(int size) {
return (unsigned char *) malloc(size*sizeof(unsigned char));
}
void delete_uint_8Array(unsigned char *a) {
free (a);
}

unsigned char get_uint_8Array(unsigned char *a, int index) {
return a[index];
}
void set_uint_8Array(unsigned char *a, int index, unsigned char val) {
a[index] = val;
}

unsigned int *new_uint_32Array(int size) {
return (unsigned int *) malloc(size*sizeof(unsigned int));
}
void delete_uint_32Array(unsigned int *a) {
free (a);
}
unsigned int get_uint_32Array(unsigned int *a, int index) {
return a[index];
}
void set_uint_32Array(unsigned int *a, int index, unsigned int val) {
a[index] = val;
}

struct arp_entry *make_arpEntryStr () {
    return (struct arp_entry *) malloc (sizeof(struct arp_entry));
}

%}
