set rev(tstdnetlib.tcl) {$Header: /home/cvs/PacketMaster/dnetlib.tcl,v 1.3 2002/09/02 15:40:46 clif Exp $}

#####################################################################
;# NAME:   dnetlib.tcl
;# AUTHOR: Clif Flynt
;# DATE:   9/2/2002
;# DESC:   Support code for assembling IP and Ethernet packets
;#         
;#         
;#
;# PARAMETERS:  Various
;#
;#
#####################################################################
# This code is written and copyrighted by Clif Flynt, 2002
#
# It is licenced for public use with these terms:
#
#  This licence agreement must remain intact with the code.
# 
#  You may use this code freely for non-commercial purposes.
#
#  You may examine and modify the code to your heart's content.
#
#  You may not claim to have authored the code (aside from modifications
#  you may have made.)
#
#  You may not distribute the code without this license agreement
#
#  You may not sell this code or distribute it with a commercial product
#      without arrangements with Noumena Corporation.
#
#  Maintenance, extension, modification, etc can be performed by:
#
#   Noumena Corporation
#   9300 Fleming Rd.
#   Dexter, MI  48130
#
#   Contact: clif@noucorp.com
#
#####################################################################


package provide dnetlib 1.0

namespace eval dnetlib {

proc list2binary {packet bufName} {
    upvar $bufName buf
    # How long is the payload
    set len [llength $packet]

    set buf [dnet::new_uint_8Array $len]

    for {set i 0} {$i < $len} {incr i} {
        dnet::set_uint_8Array $buf $i 0x[lindex $packet $i]
    }
    
    return $len
}

proc stream2bytes {data} {
  foreach d $data {
    if {[string length $d] & 0x01} {
       set d 0$d
    }
    regsub -all {..} [join $d ""] {\0 } bytes
    eval lappend bytestream $bytes
  }
  return $bytestream
}

proc checkSum {data} {
   set tot 0;
   foreach {h l} $data {
      set tot [expr $tot + 0x$l + (256 * 0x$h)]
   }
   
   while {$tot > 0xffff} {
      set tot [expr ($tot & 0xffff) + ($tot >> 16)]
   }
   set tot [expr -$tot -1]
   
   set chksum [lrange [stream2bytes [format %04x $tot]] 2 end]

   return $chksum
}

variable mnemonics
variable ipDef

array set mnemonics {
  ICMP   1
  IGMP   2
  AH	51
  TCP    6
  UDP   17
}

proc mnemonic2number {x} {
    variable mnemonics
    if {[info exists mnemonics($x)]} {
        return $mnemonics($x)
    }
    return $x
}

set Fields(IP) {
    version_len 0 1
    tos 1 1
    totalLen 2 2
    id 4 2
    flag_offset 6 2
    ttl 8 1
    protocol 9 1
    checksum 10 2
    sourceIP 12 4
    destIP 16 4
    options 20 4
}

proc doArgs {var} {
  upvar $var conversions
  set x [info level] 
  incr x -1
  
  foreach a [info args [lindex [info level $x] 0]] {
      upvar $a $a
  }
  foreach a [info args [lindex [info level $x] 0]] {
      if {[info exists conversions($a)]} {
          append packet " " [eval $conversions($a)]
      }
  }
  return [stream2bytes $packet]
}

proc insertField {headerType fieldID header value} {
    variable Fields
    
  set loc [getOffset $Fields($headerType) $fieldID]
  return [eval linsert {$header} $loc $value]
}

proc getOffset {fields id} {
    set p [lsearch $fields $id]
    if {$p < 0} {
        foreach {n l p} $fields {lappend vals $n}
        error "NO $id in $fields" "No $id in [lsort $vals]"
    }
    incr p
    return [lindex $fields $p]
}

proc getSize {fields id} {
    set p [lsearch $fields $id]
    incr p 2
    return [lindex $fields $p]
}

proc unpackIPHdr {data} {
    variable Fields
    set data [stream2bytes $data]
    foreach {n p l} $Fields(IP) {
	incr l -1
	append rtn "$n: [lrange $data $p [expr $p + $l]]\n"
    }
    return $rtn
}

proc ipPacket {version tos id flag offset ttl protocol sourceIP destIP options payload} {
   # Header length is 5 32-Bit words in basic header + 
   # Number of longwords in options
   # Total Length is header length + payload length

array set ipDef {
  version  {format "%1x%1x" $version [expr ([llength $options]/4) + 5]}
  tos      {format %02x $tos}
  id       {format %04x $id}
  flag     {format %04x [expr ($flag << 13) + ($offset & 0x1fff)]}
  ttl      {format %02x $ttl}
  protocol {format %02x [mnemonic2number $protocol]}
  sourceIP {eval format "{%02x %02x %02x %02x}" [split $sourceIP .]}
  destIP   {eval format "{%02x %02x %02x %02x}" [split $destIP .]}
  options  {set options}
}
    
  set ipHeader [doArgs ipDef]
  
  set pl [stream2bytes $payload]
  set plen [expr [llength $pl] + [llength $ipHeader] + 4]

  set ipHeader [insertField IP totalLen $ipHeader \
      [format "%02x %02x" [expr $plen >> 8] [expr $plen & 0xff] ] ]

  set chksum [checkSum $ipHeader]

  set ipHeader [insertField IP checksum $ipHeader $chksum]

  eval lappend ipHeader $payload
  return $ipHeader
}

proc icmpPacket {type code payload} {
    set tmp [eval list $type $code 0 0 $payload]
    set chksum [checkSum $tmp]
    return [eval list $type $code $chksum $payload]
}

proc pingPayload {id seq} {
    set lst [stream2bytes [format %04x $id]]
    eval lappend lst [stream2bytes [format %04x $seq]]
    for {set i 0} {$i < 56} {incr i} {
       lappend lst [format %02x $i]
    }
    return $lst
}

proc etherPacket {dest src type payload} {

    set type [list [string range $type 0 1] [string range $type 2 3]]
    
    set etherPkt [eval list [split $dest :] [split $src :] $type $payload]

    return $etherPkt
}

}
