#!/usr/local/bin/tclsh

set rev(tst.tcl) {$Header: /home/cvs/PacketMaster/tst.tcl,v 1.3 2002/09/02 15:40:46 clif Exp $}
#####################################################################
;# NAME:   tst.tcl
;# AUTHOR: Clif Flynt
;# DATE:   9/2/2002
;# DESC:   Test package for libdnet.so
;#         
;#         
;#
;# PARAMETERS:  None.  May require tweaking to send packets on
;#   your network.  (Or you can ignore checking that packets
;#   are received.
;#
#####################################################################
# This code is written and copyrighted by Clif Flynt, 2002
#
# It is licenced for public use with these terms:
#
#  This licence agreement must remain intact with the code.
# 
#  You may use this code freely for non-commercial purposes.
#
#  You may examine and modify the code to your heart's content.
#
#  You may not claim to have authored the code (aside from modifications
#  you may have made.)
#
#  You may not distribute the code without this license agreement
#
#  You may not sell this code or distribute it with a commercial product
#      without arrangements with Noumena Corporation.
#
#  Maintenance, extension, modification, etc can be performed by:
#
#   Noumena Corporation
#   9300 Fleming Rd.
#   Dexter, MI  48130
#
#   Contact: clif@noucorp.com
#
#####################################################################


load ./libdnet.so

foreach p [lsort [info commands dnet::*]] {
  puts "$p"
}
puts "SHOULD HAVE DISPLAYED A LIST OF COMMANDS"

puts ""
puts "[lsort [info commands dnet::*make*]]"
puts ""
puts "SHOULD HAVE DISPLAYED A LIST OF COMMANDS"

set a [::dnet::new_addr]
set b [::dnet::new_addr]

puts "A: $a"
dnet::addr_addr_type_set $a 2
dnet::addr_addr_type_set $b 2

dnet::addr_addr_bits_set $a 2
dnet::addr_addr_bits_set $b 2

puts "VAL: [dnet::addr_addr_type_get $a]"

set au [::dnet::addr___addr_u_get $a]
set bu [::dnet::addr___addr_u_get $b]

set ax [::dnet::addr___addr_u___data16_get $au]
set bx [::dnet::addr___addr_u___data16_get $bu]

for {set i 0} {$i < 10} {incr i} {
    dnet::set_uint_16Array $ax $i $i
    dnet::set_uint_16Array $bx $i 3
}

puts "AU: $au BU: $bu"

puts "CMP A A: [dnet::addr_cmp $a $a]"
puts "CMP A B: [dnet::addr_cmp $a $b]"

# set eth_addr [dnet::make_eth_addrStr]
set eth_addr [dnet::eth_addr_t]
 puts "ETHADDR: $eth_addr"

set e [dnet::eth_open eth0]

puts "MAC: [dnet::eth_get $e $eth_addr]"
set d [$eth_addr cget -data]

set mac ""
 for {set i 0} {$i < 6} {incr i} {
     append mac [format "%02x:" [dnet::get_uint_8Array $d $i]]
 }

puts "MAC ADDRESS for eth0: $mac"

#set e [dnet::ip_open ]

proc etherPacket {dest src type payload} {
    # How long is the payload
    set len [llength $payload]
    
    # Add 16 bytes for MAC source and destination address,
    # type and final checksum

    set etherLen [expr $len + 16]

    set buf [dnet::new_uint_8Array $etherLen]

    for {set k 6; set j 0; set i 0} {$i < 6} {incr i; incr j; incr k} {
        dnet::set_uint_8Array $buf $j 0x[lindex [split $dest :] $i]
        dnet::set_uint_8Array $buf $k 0x[lindex [split $src :] $i]
    }
    
    dnet::set_uint_8Array $buf 12 0x08
    dnet::set_uint_8Array $buf 13 0x00

    for {set j 14; set i 0} {$i < $len} {incr i; incr j} {
        dnet::set_uint_8Array $buf $j [lindex $echo_Request $i]
    }
    return $buf
}

proc list2binary {packet bufName} {
    upvar $bufName buf
    # How long is the payload
    set len [llength $packet]

    set buf [dnet::new_uint_8Array $len]

    for {set i 0} {$i < $len} {incr i} {
        dnet::set_uint_8Array $buf $i 0x[lindex $packet $i]
    }
    
    return $len
}

proc stream2bytes {data} {
    regsub -all {..} [join $data ""] {\0 } bytes
    return $bytes
}

proc checkSum {data} {
   set tot 0;
   foreach {h l} $data {
      set tot [expr $tot + 0x$l + (256 * 0x$h)]
   }
   
   while {$tot > 0xffff} {
      set tot [expr ($tot & 0xffff) + ($tot >> 16)]
   }
   set tot [expr -$tot -1]
   
   set chksum [lrange [stream2bytes [format %04x $tot]] 2 end]

   return $chksum
}

proc ipPacket {version tos id flag_offset ttl \
               protocol sourceIP destIP options payload} {

   # Header length is 5 32-Bit words in basic header + 
   # Number of longwords in options
   set hdrLen [expr ([llength $options]/4) + 5]
   set len [format "%04x" [expr (4 * $hdrLen) + [llength $payload]] ]
   set len [stream2bytes $len]

   foreach byte [split $sourceIP .] {
    lappend src [format %02x $byte]
   }

   foreach byte [split $destIP .] {
    lappend dest [format %02x $byte]
   }
   
   set id [format %04x $id]

   set ipHdr [eval list $version$hdrLen $tos $len [stream2bytes $id] \
     [stream2bytes $flag_offset] $ttl $protocol 00 00 $src $dest \
     $options ]
   
   set chksum [checkSum $ipHdr]

   set ipPkt [eval list $version$hdrLen $tos $len [stream2bytes $id] \
    [stream2bytes $flag_offset] $ttl $protocol $chksum $src $dest \
    $options $payload]

   return $ipPkt
}

proc icmpPacket {type code payload} {
    set tmp [eval list $type $code 0 0 $payload]
    set chksum [checkSum $tmp]
    return [eval list $type $code $chksum $payload]
}

proc pingPayload {id seq} {
    set lst [stream2bytes [format %04x $id]]
    eval lappend lst [stream2bytes [format %04x $seq]]
    for {set i 0} {$i < 56} {incr i} {
       lappend lst [format %02x $i]
    }
    return $lst
}

proc etherPacket {dest src type payload} {

    set type [list [string range $type 0 1] [string range $type 2 3]]
    
    set etherPkt [eval list [split $dest :] [split $src :] $type $payload]

    return $etherPkt
}

set destEther 00:E0:4C:00:14:4D
set srcEther 00:A0:CC:D1:B6:00

set echo_Request [list 45 00 00 54 00 00 40 00 40 01 \
  05 16 c0 a8 5a 40 c0 a8 5a 02 08 00 98 d9 \
  df 22 00 00 63 cf 4d 3d d5 f3 0e 00 08 09 \
  0a 0b 0c 0d 0e 0f 10 11 12 13 14 15 16 17 \
  18 19 1a 1b 1c 1d 1e 1f 20 21 22 23 24 25 \
  26 27 28 29 2a 2b 2c 2d 2e 2f 30 31 32 33 \
  34 35 36 37]

set icmp  [list 08 00 98 d9 \
  df 22 00 00 63 cf 4d 3d d5 f3 0e 00 08 09 \
  0a 0b 0c 0d 0e 0f 10 11 12 13 14 15 16 17 \
  18 19 1a 1b 1c 1d 1e 1f 20 21 22 23 24 25 \
  26 27 28 29 2a 2b 2c 2d 2e 2f 30 31 32 33 \
  34 35 36 37]


set icmp [icmpPacket 08 00 [pingPayload 255 2]]


set echo_Request [ipPacket 4 0 0 4000 40 01 192.168.9.17 192.168.9.2 "" $icmp]
puts "REQ: $echo_Request"
set packet [etherPacket $destEther $srcEther 0800 $echo_Request]
set etherLen [list2binary $packet buf]

for {set i 0} {$i < 1} {incr i} {
    dnet::eth_send $e $buf $etherLen
    after 1000
}

set echo_Request [ipPacket 4 0 0 4000 40 01 192.168.90.17 192.168.90.63 "" $icmp]
puts "REQ: $echo_Request"
set packet [etherPacket $destEther $srcEther 0800 $echo_Request]
set etherLen [list2binary $packet buf]

for {set i 0} {$i < 1} {incr i} {
    dnet::eth_send $e $buf $etherLen
    after 1000
}


